﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Net;
using Discovery.Core;

namespace Discovery.Server
{
    public partial class Form1 : Form
    {
        private string _ServerName;
        private string _EndpointAddress;

        public Form1()
        {
            this.Load += new EventHandler(Form1_Load);
            InitializeComponent();
        }

        void Form1_Load(object sender, EventArgs e)
        {
            _ServerName = ServerName.Text;
            _EndpointAddress = EndpointAddress.Text;

            ListenUDP();
        }

        private void ServerName_TextChanged(object sender, EventArgs e)
        {
            _ServerName = ServerName.Text;
        }

        private void EndpointAddress_TextChanged(object sender, EventArgs e)
        {
            _EndpointAddress = EndpointAddress.Text;
        }

        private void ListenUDP()
        {
            var broadcastAddress = new IPEndPoint(IPAddress.Any, Discovery.Core.ServiceInfo.UDPPort);
            var udpClient = new UdpClient();
            udpClient.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
            udpClient.ExclusiveAddressUse = false;
            udpClient.Client.Bind(broadcastAddress);

            UdpState s = new UdpState() { e = broadcastAddress, u = udpClient };

            udpClient.BeginReceive(new AsyncCallback(this.ReceiveCallback), s);
        }

        private void ReceiveCallback(IAsyncResult ar)
        {
            UdpState state = (UdpState)(ar.AsyncState);
            UdpClient u = state.u;
            IPEndPoint e = state.e;

            Byte[] receiveBytes = u.EndReceive(ar, ref e);
            var data = Discovery.Core.Helpers.DeserializeObject<ServiceClientInfo>(receiveBytes);

            Pings.Invoke((Action)delegate { Pings.Items.Add(String.Format("IP: {0} Port: {1}", data.IPAddress, data.Port)); });
            SendServerPingBack(data);

            u.BeginReceive(new AsyncCallback(this.ReceiveCallback), state);
        }

        private void SendServerPingBack(ServiceClientInfo client)
        {
            using (Socket sock = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp))
            {
                IPEndPoint iep1 = new IPEndPoint(IPAddress.Parse(client.IPAddress), client.Port);
                byte[] data = Discovery.Core.Helpers.SerializeObject<ServiceInfo>(new ServiceInfo(_ServerName, _EndpointAddress));
                sock.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                sock.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.Broadcast, true);
                sock.SendTo(data, iep1);
                sock.Close();
            }
        }
        
        class UdpState
        {
            public IPEndPoint e;
            public UdpClient u;
        }
    }
}
